function [pimatrix, d_index] = makePi_angles(x,y,radius,D)
% function [pimatrix, d_index] = makePi_angles(x,y,radius,D)
% Input: 
%   x, y: M-by-N matrices storing the x and y coordinates 
%    of the points. All rows of x are the same. All columns 
%    of y are the same. x and y are thus as output from e.g.
%    function "meshgrid". 
% 
%   radius: radius of the detector circle. 
%   D: number of detectors
%
% Output: 
% pimatrix: the matrix pi, of size M-by-N-by-[D(D-1)/2]. 
%    for each position x and y (first two dimension) 
%    the matrix pi stores the probabilities of detection 
%    by particular pairs of detectors (third dimension).
% 
% Order of detector pairs is stored in the d_index which is  
% a 2-by-[D(D-1)/2] matrix. 
%
% Thus to e.g. display the matrix of probability of detections 
% for a particular position (10,15), you may use: 
%
% probs = pimatrix(10,15,:); 
% dets = zeros(D,D); 
% dets(sub2ind([D,D],d_index(1,:), d_index(2,:))) = probs; 
% dets = (dets + dets') /2;
% imagesc(dets);
% colorbar

% prepare d_index: 
    
    
d_index = zeros(2,D*(D-1)/2,'single'); 
idx = 0; 
for k = 1:D
    for l = k+1:D
        idx = idx + 1; 
        d_index(:,idx) = [k;l];
    end
end

    
% compute centroid
cx = mean(x(1,:)); 
cy = mean(y(:,1)); 

[M,N] = size(x); 

pimatrix = zeros(M, N, D*(D-1)/2,'single'); 

% angles where individual detectors start:
det_angles = ((1:D) -1)/D*2*pi; 


r = radius;
% start points of individual detectors:
det_points = r*[cos(det_angles); sin(det_angles)];
det_points(1,:) = det_points(1,:)+cx; 
det_points(2,:) = det_points(2,:)+cy; 
clear cx cy det_angles idx l r radius;

for ix = 1:N
    for iy = 1:M

        pix_coords = [x(iy, ix);y(iy, ix)]; 
        
        det_pix = det_points - repmat(pix_coords,[1 D]);
        % under what angles does the point see the detectors?
        ang = atan2(det_pix(2,:), det_pix(1,:));
        
        % augment it cyclically:
        ang = ang([1:end, 1]); 

        probs = zeros(D,D);
        for k = 1:D
            cang = ang(k) + pi; % angle of start of reflected current detector
            wang = mod(ang(k+1)-ang(k), 2*pi); % angular  width 
                                               % of current det.
            tang = ang; % tang: temporary variable
            tang = mod(tang - cang +pi, 2*pi)-pi; 
            
            % compute intersection
            inters_l = max(tang(1:end-1), 0); 
            inters_p = min(tang(2:end), wang); 
            prob = (inters_p - inters_l)/(2*pi);
            prob(prob<0)=0; 
            probs(k,:)=prob;
        end
        % make probs upper-triangular:
        probs = triu(probs + probs');
        pimatrix(iy, ix, :) = probs(sub2ind([D,D],d_index(1,:), d_index(2,:)))';
    end
end

